@@ -38,10 +38,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CONFIGURE_CPPFLAGS': '-D_FORTIFY_SOURCE=2',
  'CONFIGURE_LDFLAGS': '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now',
  'CONFIG_ARGS': "'--prefix=/usr' '--enable-shared' '--with-threads' "
-                "'--with-computed-gotos' '--enable-optimizations' "
-                "'--without-lto' '--enable-ipv6' '--with-system-expat' "
-                "'--with-dbmliborder=gdbm:ndbm' '--with-system-ffi' "
-                "'--with-system-libmpdec' "
+                "'--with-computed-gotos' '--without-lto' '--enable-ipv6' "
+                "'--with-system-expat' '--with-dbmliborder=gdbm:ndbm' "
+                "'--with-system-ffi' '--with-system-libmpdec' "
                 "'--enable-loadable-sqlite-extensions' '--without-ensurepip' "
                 "'CFLAGS=-march=armv8-a -O2 -pipe -fstack-protector-strong "
                 "-fno-plt' "
